module LandTravel

use GlobalVariables
implicit none


contains

! LandSeaDestinations -> Used in MAIN
! HomogeneousLandTravel -> Usef in TravelWeights, subroutine SetTravelWeights

subroutine LandSeaDestinations
    ! For each nodes, it identifies the neighbors that are suitable for land travel
    ! Saves the number of land travel destinations and sea destinations in land_sea(2)
    ! Saves the id of the available land destinations in landNeighbors(4)
    ! Saves the id of the sea destination in seaNeigbors(4) that will be later used in
    !   case sea travel is allowed. (Or not)
    implicit none
    
    integer(4) :: i, j, landCounter, seaCounter, neighbors(8), tp
    
    do i = 1, numHabitableNodes
        i_node = habitableNodes(i)
        
        landCounter = 0
        seaCounter = 0
        Nodes(i_node)%landNeighbors = 0
        Nodes(i_node)%seaNeighbors = 0
        
        !If the node is a land node, the surroundings can only be land, coast and mountain
        if (Nodes(i_node)%node_type == 1) then
            
            !Checks the four nearest neighbors
            neighbors = Nodes(i_node)%node_neighbors
            do j = 1, 4
                tp = Nodes(neighbors(j))%node_type
                ! If the node is a land node, land and coast nodes can contribute to it
                if ((tp == 1) .or. (tp == 3)) then
                    landCounter = landCounter + 1  !Counts how many land/coast nodes contribute
                    Nodes(i_node)%landNeighbors(landCounter) = neighbors(j)  !Add the land/coast node to the list of destinations
                    
                ! Land nodes should not be next to sea nodes    
                else if (tp == 0) then
                    print*, "Land node", i_node, "is located next to a sea node", neighbors(j)
                    stop
                    
                endif
            enddo
            
        else if (Nodes(i_node)%node_type == 3) then
            neighbors = Nodes(i_node)%node_neighbors
            do j = 1, 4
                tp = Nodes(neighbors(j))%node_type
                ! If the node is a land node, land and coast nodes can contribute to it
                if ((tp == 1) .or. (tp == 3)) then
                    landCounter = landCounter + 1  !Counts how many land/coast nodes contribute
                    Nodes(i_node)%landNeighbors(landCounter) = neighbors(j)  !Add the land/coast node to the list of destinations
                
                ! If the neigbor is a sea node, we will save its id for later    
                else if (tp == 0) then
                    seaCounter = seaCounter + 1
                    Nodes(i_node)%seaNeighbors(seaCounter) = neighbors(j)
                    
                endif
                
            enddo
            
            if (seaCounter == 0) then
                print*, "Coast node", i_node, "has no sea neighbor"
                print*, "x =", Nodes(i_node)%x
                print*, "y =", Nodes(i_node)%y
                print*
                stop
            endif
            
        else
            print*, "Node", i_node, "should not be in the list of habitable nodes"
            stop
        endif
        
        !Save the number of land and sea nodes in the node info
        Nodes(i_node)%land_sea = (/landCounter, seaCounter/)       
        
    enddo
    
end subroutine LandSeaDestinations



subroutine HomogeneousLandTravel(j_node, landDest, totalDest)
    ! Sets the land destinations and relative weight of each one
    ! Sets numDestination = landDestionations, in the cases with sea travel, it will be changed later
    
    integer(4), intent(in) :: j_node, landDest, totalDest
    
    !Here we are only setting the land destinations. Coastal destinations will be set later and added to the total
    Nodes(j_node)%numDestinations = landDest

    !Set the same node as a destination and its weight: persistence                
    Nodes(j_node)%destinations(0) = j_node
    Nodes(j_node)%destinationWeight(0) = persistence
                
    !Copy the land node id's of the suitable neighbor land destinations
    Nodes(j_node)%destinations(1:landDest) = Nodes(j_node)%landNeighbors(:landDest)
    !Set each node weight, assuming that each non-mountain neighbor has the same weight
    Nodes(j_node)%destinationWeight(1:landDest) = (1-persistence)*(1./totalDest)

end subroutine HomogeneousLandTravel

end module LandTravel